package com.dotmarketing.portlets.htmlpages.business;

import java.io.Serializable;
import java.util.Date;
import java.util.List;
import java.util.Map;

import com.dotmarketing.beans.Host;
import com.dotmarketing.business.DotIdentifierStateException;
import com.dotmarketing.business.DotStateException;
import com.dotmarketing.business.query.GenericQueryFactory.Query;
import com.dotmarketing.business.query.ValidationException;
import com.dotmarketing.exception.DotDataException;
import com.dotmarketing.exception.DotSecurityException;
import com.dotmarketing.portlets.containers.model.Container;
import com.dotmarketing.portlets.folders.model.Folder;
import com.dotmarketing.portlets.htmlpageasset.model.IHTMLPage;
import com.dotmarketing.portlets.htmlpages.model.HTMLPage;
import com.dotmarketing.portlets.templates.model.Template;
import com.liferay.portal.model.User;


@Deprecated

/**
 * Provides different mechanisms to save, modify or retrieve information
 * regarding {@link HTMLPage} objects.
 *
 * @author root
 * @since 03-22-2012
 */

public interface HTMLPageAPI {

	/**
	 * Defines the different mechanisms that can be used when copying an
	 * {@link HTMLPage} object.
	 */
	public enum CopyMode {
		BLANK_HTMLPAGE, // The new html page will be created without the
						// contents
		USE_SOURCE_CONTENT, // The new html page will be created with the same
							// contents used in the source html page
		COPY_SOURCE_CONTENT; // Will copy each content used in the source html
								// page and add them to the new html page
	}

	/**
	 * 
	 */
	public static class TemplateContainersReMap {

		/**
		 * 
		 */
		public static class ContainerRemapTuple {

			private Container sourceContainer;
			private Container destinationContainer;

			public ContainerRemapTuple(Container sourceContainer,
					Container destinationContainer) {
				super();
				this.sourceContainer = sourceContainer;
				this.destinationContainer = destinationContainer;
			}

			public Container getSourceContainer() {
				return sourceContainer;
			}

			public void setSourceContainer(Container oldContainer) {
				this.sourceContainer = oldContainer;
			}

			public Container getDestinationContainer() {
				return destinationContainer;
			}

			public void setDestinationContainer(Container newContainer) {
				this.destinationContainer = newContainer;
			}

		}

		private Template sourceTemplate;
		private Template destinationTemplate;
		private List<ContainerRemapTuple> containersRemap;

		public TemplateContainersReMap(Template sourceTemplate,
				Template destinationTemplate,
				List<ContainerRemapTuple> containersRemap) {
			super();
			this.sourceTemplate = sourceTemplate;
			this.destinationTemplate = destinationTemplate;
			this.containersRemap = containersRemap;
		}

		public Template getSourceTemplate() {
			return sourceTemplate;
		}

		public void setSourceTemplate(Template oldTemplate) {
			this.sourceTemplate = oldTemplate;
		}

		public Template getDestinationTemplate() {
			return destinationTemplate;
		}

		public void setDestinationTemplate(Template newTemplate) {
			this.destinationTemplate = newTemplate;
		}

		public List<ContainerRemapTuple> getContainersRemap() {
			return containersRemap;
		}

		public void setContainersRemap(List<ContainerRemapTuple> containersReMap) {
			this.containersRemap = containersReMap;
		}

	}

	/**
	 * This an HTML copy method used for advanced copy procedures it gives you
	 * the ability to Re-map copied HTML page to a new Template and Containers
	 * and copies/moves source content to new assigned Template within it
	 * corrects Containers.
	 * <p>
	 * This methods copies an HTML page to a specified folder. Copying all
	 * source properties and permissions. Also gives you the ability to force
	 * overwrite if the page already exists at the destination.
	 * <p>
	 * You can also instruct it how the source content will be treated, gives
	 * you the options to create a blank page; no content will be associated to
	 * it or to link to same source content; which will reference same content
	 * as in the source or at last to copy the content as well; which will
	 * create copies of your source page content and attach them to the new
	 * page, if the new content requires a host or folder (dictated by its
	 * structure) then the copy will try to keep the same paths within the
	 * destination host.
	 * <p>
	 * When assigning content to the new page, you are responsible on providing
	 * a template/container re-mapping object that guides the copy on what new
	 * template will be used for the new page and host the content will be
	 * mapped to from the old containers of the source to the new containers of
	 * the re-mapping. It is your responsibility to ensure the source and
	 * destination templates and containers exists and are properly wired, this
	 * method will not validate the given templates and container re-mapping
	 * instructions.
	 * 
	 * @param htmlPage
	 * @param destination
	 * @param forceOverwrite
	 * @param copyTemplateContainers
	 * @param copyMode
	 * @param user
	 * @param respectFrontendRoles
	 * @return HTMLPage
	 * @exception DotDataException
	 * @exception DotSecurityException
	 */
	public HTMLPage copy(HTMLPage source, Folder destination,
			boolean forceOverwrite, HTMLPageAPI.CopyMode copyMode,
			TemplateContainersReMap reMapping, User user,
			boolean respectFrontendRoles) throws DotDataException,
			DotSecurityException;

	/**
	 * Copies an HTML page to a specified folder. Copies all source properties
	 * and permissions. Also gives you the ability to force overwrite if the
	 * page already exists at the destination. And you can also instruct it to
	 * copy the source page Template and Containers creating a brand new set of
	 * template and containers at the host of the destination folder an it will
	 * use those new templates for the new page.
	 * <p>
	 * You can also instruct it how the source content will be treated, gives
	 * you the options to create a blank page; no content will be associated to
	 * it or to link to same source content; which will reference same content
	 * as in the source or at last to copy the content as well; which will
	 * create copies of your source page content and attach them to the new
	 * page, if the new content requires a host or folder (dictated by its
	 * structure) then the copy will try to keep the same paths within the
	 * destination host.
	 * 
	 * @param htmlPage
	 * @param destination
	 * @param forceOverwrite
	 * @param copyTemplateContainers
	 * @param copyMode
	 * @param user
	 * @param respectFrontendRoles
	 * @return HTMLPage
	 * @exception DotDataException
	 * @exception DotSecurityException
	 */
	public HTMLPage copy(HTMLPage source, Folder destination,
			boolean forceOverwrite, boolean copyTemplateContainers,
			HTMLPageAPI.CopyMode copyMode, User user,
			boolean respectFrontendRoles) throws DotDataException,
			DotSecurityException;

	/**
	 * 
	 * Copies the source HTMLPage to a given folder. Currently this method will
	 * copy all source properties as well as permissions but will not bring the
	 * content associated with the source HTMLPage and it will reuse the same
	 * template that the source uses.
	 * <p>
	 * This method doesn't overwrite pages at the
	 * destination, so if a page with the same name already exist at the
	 * destination, it will rename the copy.
	 * 
	 * @param folderToCopyTo
	 * @return
	 * @throws DotDataException
	 */
	public HTMLPage copy(HTMLPage source, Folder folderToCopyTo, User user,
			boolean respectFrontendRoles) throws DotDataException,
			DotSecurityException;

	/**
	 * Return the working HTML Page with the specified HTML Page URL
	 * 
	 * @param htmlPageURL
	 * @param folder
	 * @return HTMLPage
	 * @throws DotDataException
	 * @throws DotStateException
	 * @throws DotSecurityException
	 */
	public HTMLPage getWorkingHTMLPageByPageURL(String htmlPageURL,
			Folder folder) throws DotStateException, DotDataException,
			DotSecurityException;

	/**
	 * 
	 * @param path
	 * @param host
	 * @return HTMLPage from a path on a given host
	 * @throws DotSecurityException
	 * @throws DotDataException
	 */
	public HTMLPage loadPageByPath(String path, Host host)
			throws DotDataException, DotSecurityException;

	/**
	 * 
	 * @param path
	 * @param host
	 * @return HTMLPage from a path on a given hostId
	 * @throws DotSecurityException
	 * @throws DotDataException
	 */
	public HTMLPage loadPageByPath(String path, String hostId)
			throws DotDataException, DotSecurityException;

	/**
	 * 
	 * @param page
	 * @param container
	 * @return true/false on whether or not a Page has content with a specificed
	 *         container
	 */
	public boolean hasContent(HTMLPage page, Container container);

	/**
	 * Use to method to get the template for the HTMLPage set on the API. This
	 * method will hit the database.
	 * 
	 * @return Template for the working version of a HTMLPage
	 * @throws DotSecurityException
	 * @throws DotDataException
	 * @throws DotStateException
	 */
	public Template getTemplateForWorkingHTMLPage(HTMLPage htmlpage)
			throws DotStateException, DotDataException, DotSecurityException;

	/**
	 * 
	 * @param folder
	 *            to get HTMLPages for
	 * @return a List of all live HTMLPages
	 * @throws DotDataException
	 * @throws DotStateException
	 * @throws DotSecurityException
	 */
	public List<HTMLPage> findLiveHTMLPages(Folder folder)
			throws DotStateException, DotDataException, DotSecurityException;

	/**
	 * 
	 * @param folder
	 *            to get HTMLPages for
	 * @return a List of all live HTMLPages
	 * @throws DotDataException
	 * @throws DotStateException
	 * @throws DotSecurityException
	 */
	public List<HTMLPage> findWorkingHTMLPages(Folder folder)
			throws DotStateException, DotDataException, DotSecurityException;

	/**
	 * Retrieves the parent folder of the given page
	 * 
	 * @param object
	 * @return
	 * @throws DotSecurityException
	 * @throws DotDataException
	 * @throws DotIdentifierStateException
	 */
	public Folder getParentFolder(HTMLPage object)
			throws DotIdentifierStateException, DotDataException,
			DotSecurityException;

	/**
	 * Retrieves the parent host of the given page
	 * 
	 * @param object
	 * @return
	 * @throws DotSecurityException
	 * @throws DotDataException
	 * @throws DotIdentifierStateException
	 */
	public Host getParentHost(IHTMLPage object)
			throws DotIdentifierStateException, DotDataException,
			DotSecurityException;

	/**
	 * Save the HTML page.
	 * 
	 * @param newHtmlPage
	 * @param template
	 * @param parentFolder
	 * @param user
	 * @param respectFrontendRoles
	 * @return HTMLPage
	 * @throws DotDataException
	 * @throws DotSecurityException
	 */
	public HTMLPage saveHTMLPage(HTMLPage newHtmlPage, Template template,
			Folder parentFolder, User user, boolean respectFrontendRoles)
			throws DotDataException, DotSecurityException;

	/**
	 * Saves the {@link HTMLPage} object in the database. Unlike the previous
	 * {@link #saveHTMLPage(HTMLPage, Template, Folder, User, boolean)}, this
	 * method will provide users the option to update an existing version of the
	 * page, instead of creating a new one.
	 * 
	 * @param newHtmlPage
	 *            - The {@link HTMLPage} that will be saved.
	 * @param template
	 *            - The {@link Template} associated to the page.
	 * @param parentFolder
	 *            - The {@link Folder} containing the page.
	 * @param user
	 *            - The current {@link User} saving the changes.
	 * @param updateVersion
	 *            - If <code>true</code>, the save process <b>will not</b>
	 *            create a new version of the page, but save the current one.
	 * @param respectFrontendRoles
	 * @return The recently saved page.
	 * @throws DotDataException
	 *             An error occurred when persisting the changes.
	 * @throws DotSecurityException
	 *             The current user does not have permissions to perform the
	 *             requested action.
	 */
	public HTMLPage saveHTMLPage(HTMLPage newHtmlPage, Template template,
			Folder parentFolder, User user, boolean updateVersion,
			boolean respectFrontendRoles) throws DotDataException,
			DotSecurityException;

	/**
	 * delete the selected HTML page
	 * 
	 * @param htmlPage
	 * @param user
	 * @param respectFrontendRoles
	 * @return boolean
	 * @throws DotSecurityException
	 * @throws Exception
	 */
	public boolean delete(HTMLPage htmlPage, User user,
			boolean respectFrontendRoles) throws DotSecurityException,
			Exception;

	/**
	 * Will search the DB. Use the SQLQueryBuilderFactory or
	 * GenericQueryBuiilderFactory to build your query object
	 * 
	 * @param query
	 * @param user
	 * @param respectFrontendRoles
	 * @return
	 * @throws DotDataException
	 * @throws ValidationException
	 * @throws ValidationException
	 * @throws DotDataException
	 */
	public List<Map<String, Serializable>> DBSearch(Query query, User user,
			boolean respectFrontendRoles) throws ValidationException,
			DotDataException;

	/**
	 * Will parse the HTMLPage and return HTMLPage rendering as a String.
	 * 
	 * @param htmlPage
	 * @return String
	 * @throws DotDataException
	 * @throws DotStateException
	 * @throws DotSecurityException
	 */
	public String getHTML(HTMLPage htmlPage, String userAgent)
			throws DotStateException, DotDataException, DotSecurityException;

	/**
	 * Will parse the HTMLPage and return HTMLPage in liveMode or workingMode
	 * rendering it as a String.
	 * 
	 * @param htmlPage
	 * @return String
	 * @throws DotDataException
	 * @throws DotStateException
	 * @throws DotSecurityException
	 */
	public String getHTML(HTMLPage htmlPage, boolean liveMode, String userAgent)
			throws DotStateException, DotDataException, DotSecurityException;

	/**
	 * Will parse the HTMLPage and return HTMLPage in liveMode or workingMode,
	 * include the URL Map Content and rendering it as a String.
	 * 
	 * @param htmlPage
	 * @return String
	 * @throws DotDataException
	 * @throws DotStateException
	 * @throws DotSecurityException
	 */
	public String getHTML(HTMLPage htmlPage, boolean liveMode,
			String contentId, String userAgent) throws DotStateException,
			DotDataException, DotSecurityException;

	/**
	 * Will parse the HTMLPage and return HTMLPage in liveMode or workingMode,
	 * include the URL Map Content and rendering it as a String.
	 * 
	 * @param htmlPage
	 * @return String
	 * @throws DotDataException
	 * @throws DotStateException
	 * @throws DotSecurityException
	 */
	public String getHTML(IHTMLPage htmlPage, boolean liveMode,
			String contentId, User user, String userAgent)
			throws DotStateException, DotDataException, DotSecurityException;

	/**
	 * Will parse the HTMLPage and return HTMLPage in liveMode or workingMode,
	 * include the URL Map Content and rendering it as a String.
	 * 
	 * @param uri
	 * @return String
	 * @throws DotDataException
	 * @throws DotStateException
	 * @throws DotSecurityException
	 */
	public String getHTML(String uri, Host host, boolean liveMode,
			String contentId, User user, String userAgent)
			throws DotStateException, DotDataException, DotSecurityException;

	/**
	 * Will parse the HTMLPage and return HTMLPage in liveMode or workingMode
	 * for a specific language ID, include the URL Map Content and rendering it
	 * as a String.
	 * 
	 * @param uri
	 * @return String
	 * @throws DotDataException
	 * @throws DotStateException
	 * @throws DotSecurityException
	 */
	public String getHTML(String uri, Host host, boolean liveMode,
			String contentId, User user, long langId, String userAgent)
			throws DotStateException, DotDataException, DotSecurityException;

	/**
	 * Retrieves the working version of a page based on its identifier
	 * 
	 * @return
	 * @throws DotSecurityException
	 * @throws DotDataException
	 */
	public HTMLPage loadWorkingPageById(String pageId, User user,
			boolean respectFrontendRoles) throws DotSecurityException,
			DotDataException;

	/**
	 * Retrieves the Live version of a page based on its identifier
	 * 
	 * @return
	 * @throws DotSecurityException
	 * @throws DotDataException
	 */
	public HTMLPage loadLivePageById(String pageId, User user,
			boolean respectFrontendRoles) throws DotSecurityException,
			DotDataException;

	/**
	 * Retrieves all html pages the given user can read
	 * 
	 * @param user
	 * @param includeArchived
	 * @param params
	 * @param hostId
	 * @param inode
	 * @param identifier
	 * @param parent
	 * @param offset
	 * @param limit
	 * @param orderBy
	 * @return
	 * @throws DotSecurityException
	 * @throws DotDataException
	 */
	public List<HTMLPage> findHtmlPages(User user, boolean includeArchived,
			Map<String, Object> params, String hostId, String inode,
			String identifier, String parent, int offset, int limit,
			String orderBy) throws DotSecurityException, DotDataException;
	
	/**
	 * Retrieves the list of HTML pages based on the specified criteria.
	 * <p>
	 * As of version 3.1, HTML pages will be represented as content. Therefore,
	 * the returned list will be composed of legacy pages ({@link HTMLPage}) and
	 * new content pages ({@link IHTMLPage}) in order to provide compatibility
	 * with existing pages.
	 * </p>
	 * 
	 * @param user
	 *            - The user in session.
	 * @param includeArchived
	 *            - If <code>true</code>, archived pages will be included in the
	 *            search. Otherwise, <code>false</code>.
	 * @param params
	 *            - A {@link Map} with additional filtering parameters.
	 * @param hostId
	 *            - The current host ID.
	 * @param inode
	 *            - The inode to search. If all results are to be included, set
	 *            to <code>null</code>.
	 * @param identifier
	 *            - The identifier to search. If all results are to be included,
	 *            set to <code>null</code>.
	 * @param parent
	 *            - If specified, represents the parent folder containing the
	 *            requested pages.
	 * @param offset
	 *            - For pagination purposes. Specifies the offset from which
	 *            data will be included in the result.
	 * @param limit
	 *            - The maximum number of results to return.
	 * @param orderBy
	 *            - The criterion used to order the collection.
	 * @return A {@link PaginatedArrayList} containing the consolidated pages
	 *         referenced with the new {@link IHTMLPage} class.
	 * @throws DotSecurityException
	 *             The current user does not have permissions to perform the
	 *             requested operation.
	 * @throws DotDataException
	 *             The page information could not be retrieved.
	 */
	public List<IHTMLPage> findIHtmlPages(User user, boolean includeArchived,
			Map<String, Object> params, String hostId, String inode,
			String identifier, String parent, int offset, int limit,
			String orderBy) throws DotSecurityException, DotDataException;

	/**
	 * 
	 * @param page
	 * @param parent
	 * @param user
	 * @param respectFrontendRoles
	 * @return
	 * @throws DotStateException
	 * @throws DotDataException
	 * @throws DotSecurityException
	 */
	public boolean movePage(HTMLPage page, Folder parent, User user,
			boolean respectFrontendRoles) throws DotStateException,
			DotDataException, DotSecurityException;

	/**
	 * 
	 * @param assetsOlderThan
	 * @return
	 * @throws DotStateException
	 * @throws DotDataException
	 */
	public int deleteOldVersions(Date assetsOlderThan)
			throws DotStateException, DotDataException;

	/**
	 * Returns the PageIds for Pages whose Templates, Containers, or Content
	 * have been modified between 2 dates even if the page hasn't been modified
	 * 
	 * @param host
	 *            Must be set
	 * @param startDate
	 *            Must be set
	 * @param endDate
	 *            Must be Set
	 * @return
	 */
	public List<String> findUpdatedHTMLPageIds(Host host, Date startDate,
			Date endDate);

	/**
	 * Returns the PageIds for Pages whose Templates, Containers, or Content
	 * have been modified between 2 dates even if the page hasn't been modified
	 * 
	 * @param host
	 *            Must be set
	 * @param pattern
	 *            url pattern e.g., /some/path/*
	 * @param include
	 *            the pattern is to include or exclude
	 * @param startDate
	 *            Must be set
	 * @param endDate
	 *            Must be Set
	 * @return
	 */
	public List<String> findUpdatedHTMLPageIdsByURI(Host host, String pattern,
			boolean include, Date startDate, Date endDate);

}
